#!/bin/bash

SQLHOST=localhost;
SQLPORT=3306;

while [[ "$#" > 0 ]]; do
    case $1 in
	-u|--user)
	    SQLUSER="$2";
	    shift
	    ;;
	-p|--password)
	    SQLPASS="$2"
	    shift
	    ;;
	-h|--host)
	    SQLHOST="$2"
	    shift
	    ;;
	-P|--port)
	    SQLPORT="$2"
	    shift
	    ;;
	-f|--filter)
	    FILTER="$2"
	    shift
	    ;;
	-h|--help)
	    cat << EOF
Usage:
 runtests.sh [options]

Options:
 -u, --user           MySQL username
 -p, --password       MySQL password
 -P, --port           MySQL port
 -h, --host           MySQL host
 -f, --filter         <matching|eq|moretap|todo|utils|charset|collation|column|constraint|engine|event|index|partition|routines|table|trigger|schemata|user|view>
EOF
	   exit 1 
	   ;;
	 *)     
	   exit 1
	   ;;
    esac;
    shift;
done


MYSQLOPTS="--disable-pager --batch --raw --skip-column-names --unbuffered"

if [[ $SQLUSER != '' ]] && [[ $SQLPASS != '' ]]; then
  MYSQLOPTS="$MYSQLOPTS -u$SQLUSER -p$SQLPASS"
fi

if [[ $SQLHOST != 'localhost' ]]; then
  MYSQLOPTS="$MYSQLOPTS --host $SQLHOST"
fi

if [[ $SQLPORT != '3306' ]]; then
  MYSQLOPTS="$MYSQLOPTS --port $SQLPORT"
fi

if [[ $FILTER == '' ]]; then
  FILTER=0
fi

MYVER=`mysql $MYSQLOPTS --execute "SELECT @@global.version" | awk -F'.' '{print $1$2}'`;

# import the full package before running the tests
# you can't use a wildcard with the source command so all version specific files need
# to be separately listed

echo "============= updating tap ============="
echo "Importing myTAP base"
mysql $MYSQLOPTS --execute 'source ./mytap.sql'

if [[ $MYVER == '56' ]]; then
    echo "Importing Version 5.6 patches";
    mysql $MYSQLOPTS --execute 'source ./mytap-table-56.sql';
fi

if [[ $MYVER == '57' ]]; then
    echo "Importing Version 5.7 patches";
    mysql $MYSQLOPTS --execute 'source ./mytap-table-57.sql';
fi

if [[ $MYVER == '80' ]]; then
    echo "Importing Version 8.0 patches";
    mysql $MYSQLOPTS --execute 'source ./mytap-table-80.sql';
fi

if [[ $FILTER != 0 ]]; then
  echo "============= filtering ============="
  echo "$FILTER"
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "matching" ]]; then
  echo "============= matching ============="
  mysql $MYSQLOPTS --database tap --execute 'source tests/matching.my'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "eq" ]]; then
  echo "============= eq ============="
  mysql $MYSQLOPTS --database tap --execute 'source tests/eq.my'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "moretap" ]]; then
  echo "============= moretap ============="
  mysql $MYSQLOPTS --database tap --execute 'source tests/moretap.my'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "todotap" ]]; then
  echo "============= todotap ============="
  mysql $MYSQLOPTS --database tap --execute 'source tests/todotap.my'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "utils" ]]; then
  echo "============= utils ============="
  mysql $MYSQLOPTS --database tap --execute 'source tests/utils.my'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "charset" ]]; then
  echo "============= character sets ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-charset.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "collation" ]]; then
  echo "============= collations ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-collation.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "column" ]]; then
  echo "============= columns ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-column.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "constraint" ]]; then
  echo "============= cconstraints ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-constraint.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "engine" ]]; then
  echo "============= engines ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-engine.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "event" ]]; then
  echo "============= events ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-event.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "index" ]]; then
  echo "============= indexes ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-index.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "partition" ]]; then
  echo "============= partitions ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-partition.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "routines" ]]; then
  echo "============= routines ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-routines.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "schemata" ]]; then
  echo "============= schemas ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-schemata.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "table" ]]; then
  echo "============= tables ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-table.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "trigger" ]]; then
  echo "============= triggers ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-trigger.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "user" ]]; then
  echo "============= users ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-user.sql'
fi

if [[ $FILTER == 0 ]] || [[ $FILTER =~ "view" ]]; then
  echo "============= views ============"
  mysql $MYSQLOPTS --database tap --execute 'source tests/test-mytap-view.sql'
fi

